from . import VK
from .config import Default
from .yandexmusic import YandexMusic
from .spotify import Spotify
import sys
import argparse
import json
from vk_api.audio import VkAudio
import logging
import vk_api
import sys
import distutils
from distutils import util
from timeit import default_timer as timer
import os
from shutil import copy2


# logging.basicConfig(filename=Default.LOG_PATH + 'running.log', filemode='w', level=logging.DEBUG,
#                     format='%(asctime)s - %(levelname)s - %(message)s')


# converting strings like 'False True 0 1' to bool
def str2bool(v):
    return bool(distutils.util.strtobool(v))


# проверяем остался ли поврежденный дата файл, если да - удаляем
# копируем и переименовываем файл-экземпляр
# если и он пропал я не виноват
def repair_template(path):
    if os.path.exists(path):
        os.remove(path)
        logging.debug(f"old data file ({path}) deleted")
    if not os.path.exists(str(path) + '.example'):
        logging.error(f"file 'dataTemplate.json.example' was not found")
    else:
        copy2(str(path) + '.example', str(path))
        logging.debug('example template successfully copied')
        return



# clears data file from past records
def clear_template(path=Default.DATATMP):
    logging.debug('cleaning template')
    with open(path) as f:
        try:
            data = json.load(f)
        except json.decoder.JSONDecodeError as jex:
            logging.error(f"Invalid JSON response, dropping to original template \n If the problem persists, check connection to desired service")
            logging.error(jex)
            repair_template(path)
            return
        for i in data:
            for j in data[i]:
                data[i][j].clear()
        with open('dataTemplate.json', 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=2, ensure_ascii=False)
        return data


# Parser for command-line options
def process_args(args, defaults):

    parser = argparse.ArgumentParser()
    parser.prog = 'MMM'

    subparsers = parser.add_subparsers(title='subcommands',
                                       description='valid subcommands',
                                       help='description')

    parser_base = argparse.ArgumentParser(add_help=False)

    parser_base.set_defaults(scope=defaults.SCOPE)  # область необходимых разрешений для работы со spotify API

    parser_base.add_argument('--log-path', dest="log_path",
                             metavar=defaults.LOG_PATH,
                             type=str, default=defaults.LOG_PATH,
                             help=('log file path (default: %s)'
                                   % (defaults.LOG_PATH)))


    # ОБЩИЕ аргументы
    parser_model = argparse.ArgumentParser(add_help=False)
    parser_model.add_argument('--data-path', dest='data_path', type=str, default=defaults.DATATMP,
                            help=('path+filename to data template(default: %s)' % (defaults.DATATMP)))

    parser_model.set_defaults(data_path=defaults.DATATMP)

    parser_model.add_argument('--playlists', dest='playlists',
                              action='store_true', default=defaults.PLAYLIST,
                              help=('include playlists as well (default: %s)' % (defaults.PLAYLIST)))

    parser_model.add_argument('--clean-plate', dest='clean_plate',
                                action='store_true', default=defaults.CLEAN_PLATE,
                                help='Delete all past records in dataTemplate')

    parser_model.add_argument('--artists', dest='artists',
                              action='store_true', default=defaults.ARTISTS,
                              help=('include artists as well (default: %s)' % (defaults.ARTISTS)))

    parser_model.add_argument('--albums', dest='albums',
                              action='store_true', default=defaults.ALBUMS,
                              help=('include albums as well (default: %s)' % (defaults.ALBUMS)))

    parser_model.add_argument('--playlists-l', dest='playlists_l',
                               nargs='+',
                              default=defaults.PLAYLIST_L,
                              help=('include albums as well (default: %s)' % (defaults.PLAYLIST_L)))

    parser_model.add_argument('--alltracks', dest='alltracks',
                              action='store_true',
                              default=Default.ALLTRACKS,
                              help=('include albums as well (default: %s)' % (defaults.ALLTRACKS)))

    parser_model.add_argument('--source-user', dest='source_user',
                              type=str, nargs=1, required=True, help='login for source site')

    parser_model.add_argument('--source-pass', dest='source_pass',
                              type=str, nargs=1, required=True, help='password for source site')

    # Аргументы для ЭКСПОРТА данных в файл
    export_parser = subparsers.add_parser('export', parents=[parser_base, parser_model],
                                          help='export music to json file')
    export_parser.set_defaults(phase='export')

    export_parser.add_argument('-s', '--source', dest='source', required=True, choices=['vk', 'ym', 'sp'], type=str,
                            help='service_name to fetch music from')


    # Аргументы для ПОЛНОГО ПОСЛЕДОВАТЕЛЬНОГО импорта<=>экспорта
    run_parser = subparsers.add_parser('run', parents=[parser_base, parser_model],
                                          help='run full program from the beginning till the end')
    run_parser.set_defaults(phase='run')

    run_parser.add_argument('-s', '--source', dest='source', required=True, choices=['vk', 'ym', 'sp'], type=str,
                            help='service_name to fetch music from')

    run_parser.add_argument('-t', '--target', required=True, dest='target', choices=[
                               "ym", "sp"], type=str, help='service_name to export music to')

    run_parser.add_argument('--target-user', dest='target_user',
                              type=str, nargs=1, required=True, help='login for target site')

    run_parser.add_argument('--target-pass', dest='target_pass',
                              type=str, nargs=1, required=True, help='password for target site')

    parameters = parser.parse_args(args)
    return parameters


def main(args=None):

    if args is None:
        args = sys.argv[1:]
    parameters = process_args(args, Default)
    data_path = parameters.data_path

    if not os.path.exists(parameters.log_path):
        os.mkdir(parameters.log_path)

    logging.basicConfig(filename=parameters.log_path + 'running.log', filemode='w', level=logging.DEBUG,
                        format='%(asctime)s - %(levelname)s - %(message)s')

    # Проверяем существует ли дата файл и наличие содержимого
    # Таким образом пресекаем импорт до первого экспорта
    try:
        with open(data_path) as f:
            if parameters.clean_plate:
                logging.warning(f"'clean-plate!!!!' flag is up. Reseting data file")
                clear_template(data_path)
            data = json.load(f)
    except FileNotFoundError as e:
        data = {
            "YM": {
                "artists": [],
                "albums": [],
                "playlists": {},
                "alltracks": []
            },
            "SP": {
                "artists": [],
                "albums": [],
                "playlists": {},
                "alltracks": []
            },
            "VK": {
                "artists": [],
                "albums": [],
                "playlists": {},
                "alltracks": []
            }
        }
        with open('dataTemplate.json', 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=4, ensure_ascii=False)

    if parameters.phase == 'export' or parameters.phase == 'run':
        if parameters.source == "vk":
            vkaudio = VK.get_auth(login=parameters.source_user,
                                  password=parameters.source_pass)

            # Сразу вызываем экспорт плейлиста, так как вк кал и больше ничего нельзя
            selectExport(imModel=vkaudio, imPhase=parameters.phase,
                        parameters=parameters, imSource=parameters.source, datafile=data)

        elif parameters.source == 'ym':
            imModel = YandexMusic(
                parameters.source_user, parameters.source_pass, data, playlists_l=parameters.playlists_l)
            selectExport(imModel, imPhase=parameters.phase, parameters=parameters)

        elif parameters.source == 'sp':
            imModel = Spotify(parameters.source_user,
                                parameters.scope, data)
            selectExport(imModel, imPhase=parameters.phase, parameters=parameters)



# Импорт данных с учетом конфига/командных аргументов
def selectImport(imModel, parameters):
    if parameters.playlists or parameters.playlists_l:
        imModel.import_playlists()
    if parameters.alltracks:
        imModel.import_alltracks()
    if parameters.artists:
        imModel.import_artists()
    if parameters.albums:
        imModel.import_albums()
    return "SELECT IMPORT SUCCEDED"


# Экспорт данных с учетом конфига/командных аргументов
# Фазы экспорта и полного пробега начинаются в одной точке
# При экспорте данные просто пишутся в файл
# При выполнении всей программы (run) последовательно выполняется сначала экспорт из выбранной платформы,
# сразу за ним вызывается импорт на целевую платформу (selectImport)
# В качестве выхода - строка результат выполнения
# imModel - с API какого сервиса работаем в данный момент [vk,sp,ym]
# imSource - Откуда берем музыку ---> записывается в файл
# imPhase - Какую часть кода выполнять:
#                                   [export - получаение данных и запись,
#                                   run - получение => применение]

def selectExport(imModel, imPhase, parameters, imSource=None, datafile=None):
    if parameters.playlists or parameters.playlists_l:
        if imSource == 'vk':
            data = VK.export_playlists(imModel, datafile, parameters.playlists_l)
        else:
            imModel.export_playlists()
    if parameters.alltracks:
        if imSource == 'vk':
            logging.error('vk alltracks export unavailable')    #БАГ API
            sys.exit()
        else:
            imModel.export_alltracks()
    if parameters.artists:
        imModel.export_artists()
    if parameters.albums:
        imModel.export_albums()


    # извлекаем данные для всех, кроме вк. У него лишняя хромосома
    if imSource != 'vk':
        data = imModel.export_data


    # Если задача экспорта - пишем в файл и останавливаемся
    # В противном случае (run) сразу начинаем импорт
    # на всякий случай возвращаем статус выполнения каждого шага
    if imPhase == 'export':
        with open('dataTemplate.json', 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=4, ensure_ascii=False)
            logging.debug('EXPORT PHASE SUCCESS')
            return "SELECT EXPORT SUCCEDED"
    else:
        with open('dataTemplate.json', 'w', encoding='utf-8') as f:
            json.dump(data, f, indent=4, ensure_ascii=False)
        if parameters.target == 'sp':
            imModel = Spotify(parameters.target_user,
                    parameters.scope, data, source=parameters.source)
        elif parameters.target == 'ym':
            imModel = YandexMusic(
                parameters.target_login, parameters.target_pass, data, playlists_l=parameters.playlists_l, source=parameters.source)
        status = selectImport(imModel, parameters)
        if status is not None:
            return "FULL RUN SUCCEDED"
        else:
            logging.debug('EXPORT PHASE SUCCESS')
            return "ERROR WRONG ARGUMENTS"


if __name__ == '__main__':
    main()



"""
    Default parameters
    Configure parameters for preferred result.
"""
from os import environ
from dotenv import load_dotenv
load_dotenv()

class Default(object):

    SP_CLIENT_ID = environ.get('SP_CLIENT_ID') or None
    SP_CLIENT_SECRET = environ.get('SP_CLIENT_SECRET') or None
    SCOPE = "user-library-read user-library-modify playlist-read-private playlist-modify-private"

    PLAYLIST = False
    PLAYLIST_L = []
    ARTISTS = False
    ALBUMS = False
    ALLTRACKS = False
    DATATMP = './dataTemplate.json'
    LOG_PATH = './logs/'
    CLEAN_PLATE = True



import sys
import spotipy
import spotipy.util as util
from .config import Default
import logging
import collections
import json
import pprint
import re

# logging.basicConfig(level=logging.DEBUG,
                    # format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logging.getLogger('spotipy.client').setLevel(logging.ERROR)
logging.getLogger('urllib3.connectionpool').setLevel(logging.ERROR)


class Spotify(object):
    def __init__(self, username, scope, data, source=None):
        self.export_data = data
        self.username = username
        self.source = source
        self.scope = 'user-library-modify user-library-read user-follow-read user-follow-modify playlist-read-private playlist-modify-public playlist-modify-private playlist-read-collaborative'
        token = util.prompt_for_user_token(
            self.username,
            self.scope,
            client_id=Default.SP_CLIENT_ID,
            client_secret=Default.SP_CLIENT_SECRET,
            redirect_uri='http://example.com')
        if token:
            logging.debug(f'token OK: (token)')
            self.sp = spotipy.Spotify(auth=token)
        else:
            logging.error(f"Can't get token for {self.username}")

    def show_tracks(self, list_type, tracks, playlist_name=None):
        for i, item in enumerate(tracks['items']):
            track = item['track']
            print("   %d %32.32s %s" %
                  (i, track['artists'][0]['name'], track['name']))
            if playlist_name:
                self.export_data["SP"][list_type][playlist_name].append(
                    [track['name'], track['artists'][0]['name']])
            else:
                self.export_data["SP"][list_type].append(
                    [track['name'], track['artists'][0]['name']])

    def export_playlists(self):
        playlists = self.sp.user_playlists(self.username)
        logging.debug(f"START export onwned playlists")
        for playlist in playlists['items']:
            if playlist['owner']['id'] == self.username.lower():    # ONLY OWNED BY USER
                logging.debug(f"START export playlists '{playlist['name']}'")
                self.export_data["SP"]["playlists"][playlist['name']] = []
                results = self.sp.playlist(
                    playlist['id'], fields="tracks,next")
                tracks = results['tracks']
                logging.debug('showing tracks')
                self.show_tracks('playlists', tracks, playlist['name'])
                while tracks['next']:
                    tracks = self.sp.next(tracks)
                    self.show_tracks('playlists', tracks, playlist['name'])
                logging.debug(
                    f"DONE export playlist '{playlist['name']}'; {playlist['tracks']['total']}:{len(self.export_data['SP']['playlists'][playlist['name']])}")

    def export_alltracks(self):
        self.export_data["SP"]["alltracks"] = []
        tracks = self.sp.current_user_saved_tracks(limit=50)
        logging.debug(f"START export playlists 'My favorites'")
        self.show_tracks('alltracks', tracks)
        while tracks['next']:
            tracks = self.sp.next(tracks)
            self.show_tracks('alltracks', tracks)
        logging.debug(
            f"DONE export playlist 'My favorites', Total: {len(self.export_data['SP']['alltracks'])}")

    def export_artists(self):
        self.export_data["SP"]["artists"] = []
        results = self.sp.current_user_followed_artists()
        artists = results['artists']
        for item in artists['items']:
            self.export_data["SP"]["artists"].append(item['name'])
        while artists['next']:
            logging.debug('Exporting next page of artists...')
            artists = self.sp.next(artists)['artists']
            for item in artists['items']:
                print(item['name'])
                self.export_data["SP"]["artists"].append(item['name'])
        logging.debug(
            f"DONE export artists, Total: {len(self.export_data['SP']['artists'])}")

    def export_albums(self):
        albums = self.sp.current_user_saved_albums()
        for item in albums['items']:
            album_title = item['album']['name']
            artist_name = item['album']['artists'][0]['name']
            tracks = item['album']['tracks']['items']
            self.export_data["SP"]["albums"].append({
                                                    "title": album_title,
                                                    "artist": artist_name,
                                                    "tracks_count": len(tracks)
                                                    })
        while albums['next']:
            logging.debug('Exporting next page of albums...')
            albums = self.sp.next(albums)
            for item in albums['items']:
                album_title = item['album']['name']
                artist_name = item['album']['artists'][0]['name']
                tracks = item['album']['tracks']['items']
                self.export_data["SP"]["albums"].append({
                    "title": album_title,
                    "artist": artist_name,
                    "tracks_count": len(tracks)
                })
        logging.debug(
            f"DONE export albums, Total: {len(self.export_data['SP']['albums'])}")

    def unfollow_all(self):
        results = self.sp.current_user_followed_artists()
        artist_tuple = [(item['id'], item['name'])
                        for item in results['artists']['items']]
        artist_ids = [item['id'] for item in results['artists']['items']]
        print(artist_tuple)
        print(artist_ids)
        self.sp.user_unfollow_artists(artist_ids)

    def import_playlists(self):
        for title, item in self.export_data[self.source.upper()]["playlists"].items():
            track_ids = []
            new_playlist = self.sp.user_playlist_create(
                user=self.username, name=title + f'({self.source.upper()})')
            playlist_id = new_playlist['id']
            playlist_name = new_playlist['name']
            logging.debug(f"CREATED new playlist '{playlist_name}'")
            for track in item:
                results = self.sp.search(
                    q=track[0] + ' ' + track[1], type='track')
                tracks = results['tracks']
                if len(tracks['items']) != 0:
                    track_ids.append(tracks['items'][0]['id'])
                    logging.debug(
                        f"ADDED track '{tracks['items'][0]['name']}' to playlist '{playlist_name}'")
                else:
                    logging.error(
                                f"ERROR COULDNT FIND TRACK '{track[0]}-{track[1]}'")

            while len(track_ids) > 100:
                self.sp.user_playlist_add_tracks(
                    self.username, playlist_id, track_ids[:100])
                track_ids = track_ids[100:]

            self.sp.user_playlist_add_tracks(
                self.username, playlist_id, track_ids)
            logging.debug(
                f"DONE {len(track_ids)} tracks added to playlist '{playlist_name}'")

        logging.debug(f"DONE import playlists")

    def import_artists(self):
        artists = self.export_data[self.source.upper()]["artists"]
        artist_ids = []
        logging.debug(f"Total artist count: {len(artists)}")
        for artist in artists:
            logging.debug(f"START search artist '{artist}'")
            results = self.sp.search(
                q=artist + ' ', type='artist', limit=50, market='RU')
            items = results['artists']['items']
            logging.debug(
                f"TOTAL ARTIST BY NICKNAME {artist} -- {len(results['artists']['items'])}")

            # unique name or translit from russian --> No need for verification
            if len(items) != 0:
                curr_artist = items[0]
                artist_ids.append(curr_artist['id'])
                logging.debug(
                    f"DONE search artist '({curr_artist['name']}-{artist}):{curr_artist['id']}'")
                continue

            elif len(items) == 0:
                logging.debug(
                    f"EROR while search for {results}, {len(results['artists']['items'])}")
        logging.debug(f"DONE artists count: {len(artist_ids)}")
        self.sp.user_follow_artists(artist_ids)



    def import_albums(self):
        import_albums = self.export_data[self.source.upper()]["albums"]
        for item in import_albums:
            album_title = item["title"]
            artist_name = item["artist"]
            album_count = item["tracks_count"]
            try:
                results = self.sp.search(
                    q='album:' + album_title + ' ' + artist_name, type='album')
                search_artist = results['albums']['items'][0]['artists'][0]['name']
                search_title = results['albums']['items'][0]['name']
                search_uri = results['albums']['items'][0]['uri']
                search_tracks = results['albums']['items'][0]['total_tracks']
            except IndexError:
                logging.error(
                    f"COUDNT FIND ALBUM '{album_title}-{artist_name}' ON SPOTIFY")
                continue

            # Verification
            # сравниваем тайтл, имя артиста и количество песен в альбоме (source vs target)
            if search_title.casefold() == album_title.casefold() and search_artist.casefold() == artist_name.casefold() and int(album_count) == int(search_tracks):
                logging.debug(f"Album '{album_title}-{artist_name}' OK")
                # add album if not already added
                self.sp.current_user_saved_albums_add(albums=[search_uri])
                logging.debug(
                    f"DONE album '{album_title}-{artist_name}' added")



    def import_alltracks(self):
        track_ids = []
        alltracks = self.export_data[self.source.upper()]["alltracks"]
        new_playlist = self.sp.user_playlist_create(
                    user=self.username, name=f'All tracks({self.source.upper()})')
        playlist_id = new_playlist['id']
        playlist_name = new_playlist['name']
        logging.debug(f"CREATED new playlist '{playlist_name}'")
        for track in alltracks:
            results = self.sp.search(
                q=track[0] + ' ' + track[1] + ' ', type='track')
            tracks = results['tracks']
            if len(tracks['items']) != 0:
                track_ids.append(tracks['items'][0]['id'])
                logging.debug(
                    f"ADDED track '{tracks['items'][0]['name']}' to playlist '{playlist_name}'")

            elif len(tracks['items']) == 0:
                logging.error(
                    f"ERROR COULDNT FIND TRACK '{track[0]}-{track[1]}'")
        while len(track_ids) > 100:
            self.sp.user_playlist_add_tracks(
                self.username, playlist_id, track_ids[:100])
            track_ids = track_ids[100:]

        self.sp.user_playlist_add_tracks(
            self.username, playlist_id, track_ids)
        logging.debug(
            f"DONE {len(track_ids)} tracks added to playlist '{playlist_name}'")



# -*- coding: utf-8 -*-
import collections
from yandex_music.client import Client
from .config import Default
import json
import logging
import requests
import time


# logging.basicConfig(level=logging.DEBUG,
                    # format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logging.getLogger('yandex_music').setLevel(logging.ERROR)


class YandexMusic(object):
    def __init__(self, login, password, export_data, playlists_l, source=None):
        self.failed = []
        self.my_id = None
        self.client = Client.from_credentials(login, password)
        self.export_data = export_data
        self.playlists_l = playlists_l
        self.source = source


    # Проверяем существует ли нужный плейлист. Нет - создаем с нужным тайтлом и возвращаем ID, Да - просто возвращаем ID
    def check_playlist(self, playlist_title):
        logging.debug(f"Checking if playlist '{playlist_title}' exists")
        my_playlists = self.client.users_playlists_list()
        for playlist in my_playlists:
            if playlist['title'] == playlist_title:
                logging.debug(f'playlist "{playlist_title}" exists; its id: "{playlist["kind"]}"')
                return (playlist['kind'], playlist['revision'])

        playlist_new = self.client.users_playlists_create(title=playlist_title)
        logging.debug(f"playlist '{playlist_title}' not found; created with id: '{playlist_new['kind']}'")
        return (playlist_new['kind'], playlist_new['revision'])



    # Поиск информации об исполнителе. Возвращает json файл: имя, ID, все альбомы, все треки
    def find_artist(self, artist):
        info_dict = collections.defaultdict(dict)
        artist_info = self.client.search(text=artist, nocorrect=True, type_='artist')['artists']['results'][0]
        artist_id = artist_info['id']
        artist_albums = self.client.artists_direct_albums(artist_id=artist_id)['albums']
        artist_tracks = self.client.artists_tracks(artist_id=artist_id, page_size=100)['tracks']
        info_dict['artist_name'] = artist.lower()
        info_dict['artist_id'] = artist_id
        for album in artist_albums:
            info_dict['artist_albums'][album['title'].lower()] = {
                                                        'album_id': album['id'],
                                                        'track_count': album['track_count']}
        for track in artist_tracks:
            info_dict['artist_tracks'][track['title'].lower()] = [track['id'], track['albums'][0]['title'].lower(), track['albums'][0]['id']]

        with open('moo.json', 'w', encoding='utf-8') as f:
            json.dump(info_dict, f, indent=4, ensure_ascii=False)
        return info_dict

    def import_albums(self):
        import_albums = self.export_data[self.source.upper()]["albums"]
        for item in import_albums:
            album_title = item["title"]
            artist_name = item["artist"]
            album_count = item["tracks_count"]
            try:
                search_text = album_title + ' ' + artist_name
                results = self.client.search(text=search_text, nocorrect=True, type_="album")['albums']['results'][0]
                search_title = results.title
                search_artist = results.artists[0].name
                search_count = results.track_count
            except IndexError:
                logging.error(
                    f"COUDNT FIND ALBUM '{album_title}-{artist_name}' ON YANDEX MUSIC")
                continue
            if search_title.casefold() == album_title.casefold() and search_artist.casefold() == artist_name.casefold() and int(album_count) == int(search_count):
                logging.debug(f"Album '{album_title}-{artist_name}' OK")
                album_id = results.id
                # add album if not already added
                self.client.users_likes_albums_add(album_ids=album_id)
                logging.debug(
                    f"DONE album '{album_title}-{artist_name}' added")

    def import_artists(self):
        artists = self.export_data[self.source.upper()]["artists"]
        for artist in artists:
            logging.debug(f"START search artist '{artist}'")
            try:
                results = self.client.search(text=artist, type_='artist', nocorrect=False)['artists']['results'][0]
                search_artist = results.name # for verification in future
                logging.debug(f"found search_artist {search_artist}")
                artist_id = results.id
                self.client.users_likes_artists_add(artist_ids=artist_id)
                logging.debug(f"{search_artist} ADDED OK")
            except (IndexError, TypeError) as e:
                logging.error(
                    f"COUDNT FIND ARTIST '{artist}' ON YANDEX MUSIC")
                logging.error(e)
                continue

    def import_playlists(self):
        import_tracks = ''
        url='https://music.yandex.ru/handlers/import.jsx'
        playlists = self.export_data[self.source.upper()]["playlists"]
        for title, tracks in playlists.items():
            playlist_id, playlist_rev = self.check_playlist(title)
            for item in tracks:
                import_tracks += ' '.join(item) + '\n'
            string_of_songs=import_tracks.replace(' ','+')
            json_values={
                'content':string_of_songs
            }
            r1_import=requests.post(url,json=json_values)

            # GET
            id_import=json.loads(r1_import.text)['importCode']
            params={
                'code':id_import
            }
            resp=requests.get(url=url, params=params)
            resp = json.loads(resp.text)
            logging.debug(f"{resp['status']}")

            while True:
                if resp['status'] == 'in-progress':
                    logging.debug(f"{resp['status']}")
                    time.sleep(2)
                    r2_import=requests.get(url=url, params=params)
                    resp = json.loads(r2_import.text)
                elif resp['status'] == 'done':
                    logging.debug(f"{resp['status']}")
                    trackIds = resp['trackIds']
                    print(trackIds)
                    break

            for trackId in trackIds:
                self.client.users_playlists_insert_track(playlist_id, trackId.split(':')[0], trackId.split(':')[1], revision=playlist_rev)
                playlist_rev += 1


    def import_alltracks(self):
        alltracks = self.export_data[self.source.upper()]["alltracks"]
        import_tracks = ''
        url='https://music.yandex.ru/handlers/import.jsx'
        for item in alltracks:
            import_tracks += ' '.join(item) + '\n'
        string_of_songs=import_tracks.replace(' ','+')
        json_values={
            'content':string_of_songs
        }
        r1_import=requests.post(url,json=json_values)

        # GET
        id_import=json.loads(r1_import.text)['importCode']
        url='https://music.yandex.ru/handlers/import.jsx'
        params={
            'code':id_import
        }
        resp=requests.get(url=url, params=params)
        resp = json.loads(resp.text)
        logging.debug(f"{resp['status']}")

        while True:
            if resp['status'] == 'in-progress':
                logging.debug(f"{resp['status']}")
                time.sleep(2)
                r2_import=requests.get(url=url, params=params)
                resp = json.loads(r2_import.text)
            elif resp['status'] == 'done':
                logging.debug(f"{resp['status']}")
                trackIds = resp['trackIds']
                print(trackIds)
                break
        playlist_id, playlist_rev = self.check_playlist(f'Все треки ({self.source.upper()})')
        logging.debug(f"added 'Все треки ({self.source.upper()})'")
        for trackId in trackIds:
            self.client.users_playlists_insert_track(playlist_id, trackId.split(':')[0], trackId.split(':')[1], revision=playlist_rev)
            playlist_rev += 1
        # self.client.users_likes_tracks_add(track_ids=trackIds)



    def export_artists(self):
    # export liked artists list, skip if 'False'
        liked_artists = self.client.users_likes_artists()
        self.export_data["YM"]["artists"] = []
        for like in liked_artists:
            artist_id = like.artist.id
            artist_name = like.artist.name
            self.export_data["YM"]["artists"].append(artist_name)



    def export_albums(self):
        # export liked albums of certain artist(manually defined or ALL), skip if list is empty
        logging.debug('START export albums')
        liked_albums = self.client.users_likes_albums()
        self.export_data["YM"]["albums"] = []
        for like in liked_albums:
            if like.album.type == 'podcast':
                logging.warning(f"SKIP track with type: '{like.album.type}'")
                continue
            logging.debug(f"START export album '{like.album.title}'")
            album_title = like.album.title
            artist_name = like.album.artists[0].name
            track_count = like.album.track_count

            self.export_data['YM']['albums'].append({"title": album_title, "artist": artist_name, "tracks_count": track_count})
            logging.debug(f"DONE export album '{like.album.title}'")



    # export your playlists (manually defined or favorites), skip if list is empty
    def export_alltracks(self):
        logging.debug('START export alltracks')
        liked_tracks = self.client.users_likes_tracks().tracks
        track_count = len(liked_tracks)
        self.export_data["YM"]["alltracks"] = []
        for like in liked_tracks:
            track_title = like.track.title
            artist_name = like.track.artists[0].name
            self.export_data["YM"]["alltracks"].append([artist_name, track_title])
            logging.debug(f"{artist_name} - {track_title} OK")
        logging.debug(f"DONE export playlist 'My favorites' TOTAL: {track_count}")



    def export_playlists(self):
        my_playlists = {playlist.title: playlist.kind for playlist in self.client.users_playlists_list() if playlist}
        logging.debug('my playlists: {}'.format([item for item in my_playlists.keys()]))
        logging.debug('specified playlists: {}'.format(self.playlists_l))
        if not self.playlists_l:
            pass
        else:
            my_playlists = {key:val for key,val in my_playlists.items() if key in  self.playlists_l}
        for item, playlist_id in my_playlists.items():
            try:
                playlist = self.client.users_playlists(playlist_id)[0].tracks
            except KeyError:
                logging.error(f"FAILED TO FIND PLAYLIST '{item}'")
                continue
            self.export_data["YM"]["playlists"][item] = []
            logging.debug(f"START export playlist '{item}'")
            for like in playlist:
                if 'podcast' in like.track.type:
                    logging.warning(f"SKIP track with type: '{like.track.type}'")
                    continue
                track_title = like.track.title
                artist_name = like.track.artists[0].name
                self.export_data["YM"]["playlists"][item].append([artist_name, track_title])
                logging.debug(f"{artist_name} - {track_title} OK")
            logging.debug(f"DONE export playlist '{item}'")




    # -*- coding: utf-8 -*-
import collections

from yandex_music.client import Client

import vk_api
from vk_api.audio import VkAudio
from .config import Default
import json
import re
import logging

# logging.basicConfig(level=logging.DEBUG,
                    # format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')


def get_auth(login, password):
    # init vk API session with given credentials
    vk_session = vk_api.VkApi(login, password)
    # Attempt to login into account
    try:
        vk_session.auth()
    except vk_api.AuthError as error_msg:
        logging.error('Two factor auth needed. If you have it on purpose - disable. \n Otherwise try waiting some time')
        return error_msg
    logging.debug('Auth OK')
    vkaudio = VkAudio(vk_session)
    return vkaudio


def export_playlists(vkaudio, datafile, specplaylist=Default.PLAYLIST_L):
    # Decide what we need to import
    logging.debug('GETPLAYLIST True')
    all_playlist = vkaudio.get_albums()
    if specplaylist:
        logging.debug('PLAYLISTS_L TRUE')
        all_playlist = [album for album in all_playlist if album['title'].lower() in specplaylist]
        logging.debug('found match {}'.format(len(all_playlist)))             # if user chose only specific playlists we check if names eq- with list comprehension
    logging.debug('get playlist tracks START')
    for item in all_playlist:
        playlist_title = item['title'].lower()        # get track list of each chosen playlist -> edit strings -> convert to dict
        logging.debug('GET PLAYLIST: {}'.format(playlist_title))
        datafile["VK"]["playlists"][playlist_title] = []
        tr_list = vkaudio.get(owner_id=item['owner_id'], album_id=item['id'], access_hash=item['access_hash'])
        for n, track in enumerate(tr_list, 1):
            artist_name = parse_artist(track['artist'].lower())     # delete disturbing symbols/spaces etc. cuz they'll interfere with further str. comparsion
            track_title = track['title'].lower()
            datafile["VK"]["playlists"][playlist_title].append([artist_name, track_title])
            logging.debug('     +{0} - {1}'.format(artist_name, track_title))
        logging.debug('PLAYLIST TOTAL: {}'.format(len(datafile["VK"]["playlists"][playlist_title])))
    logging.debug('get playlist tracks DONE')
    return datafile


# СУКИ ВСЁ СЛОМАЛИ НИЧЕ НЕ РАБОТАЕТ(
def export_alltracks(vkaudio, datafile):
    data = []
    logging.debug('GETPLAYLIST False. exporting all tracks')
    for item in vkaudio.get():            # get tracklist and convert it to dict
        artist_name = parse_artist(item['artist'].lower())
        track_title = item['title'].lower()
        logging.debug(track_title)
        data.append([artist_name, track_title])
    logging.debug('all tracks export DONE. stats: list of list - {0}'.format(len(data)))
    return data




def parse_artist(artist_name):
    spec_symbols = re.split(', | \\.feat | feat\\. | ft | ft\\. | \\.ft| x | & ', artist_name)
    artist_name = spec_symbols[0].strip()
    return artist_name




import setuptools

REQUIRED_PACKAGES = ['vk-api', 'yandex-music', 'spotipy']

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Move-My-Music",
    version="0.0.2dev7",
    author="Max Medvedev",
    author_email="medve.mk@gmail.com",
    description="Script to migrate music between platforms",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Addic7edBoy/MoveMyMusic",
    packages=setuptools.find_packages(),
    install_requires=[
        "vk-api==11.8.0",
        "yandex-music==0.1.2",
        "spotipy==2.13.0",
        "six",
        "beautifulsoup4",
        "requests",
        "python-dotenv"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': ['MMM=MoveMyMusic.__main__:main'],
    }
)
